/*
 * rpmets.	Transactional RPM
 *
 * Extend the rpmlib to handle more than one type of actions over
 * the same transaction set
 *
 * Rafael A. Garcia Leiva (angel.leiva@uam.es)
 * Universidad Autonoma de Madrid
 *
 */

#include <fcntl.h>
#include <string.h>
#include <rpm/rpmlib.h>
#include <rpm/rpmts.h>
#include <rpm/rpmdb.h>
#include <rpm/rpmio.h>
#include <rpm/rpmmacro.h>

#define MAXMACROSIZE 1024	/* maximum size for rpm's macros */

void rpmEts_AddInstallElement(rpmts ts, const char *package, int upgrade);
static void *notify_CB(const void *arg, const rpmCallbackType what,
                       const unsigned long amount, const unsigned long total,
                       fnpyKey key, void *data);
void error_CB(void);

/* global variables to handle command line parameters */
extern int verbose, debug;
extern char *rcfile, *rootdir, *dbpath;
extern rpmInstallInterfaceFlags interface_flags;
extern rpmtransFlags rpmts_flags;
extern rpmprobFilterFlags rpmprob_filter;
extern rpmVSFlags vsflags;

/*
 * Create and intilialize a new transaction set
 * Returns the new transaction set
 */
rpmts rpmEtsInit(void) {

    rpmts ts;

    if (verbose) {
        printf("Initializing transaction set\n");
    }

    /* read configuration file pointed by rcfile, if rcfile is NULL
     * default configuration file will be used instead */
    if (rpmReadConfigFiles(rcfile, NULL)) {
        printf("Failed to read RPM config file\n");
        exit(1);
    }

    ts = rpmtsCreate();

    /* set notify and error callback functions */
    rpmtsSetNotifyCallback(ts, notify_CB, NULL);
    rpmErrorSetCallback(error_CB);

    if (rootdir) {
        /* use an alternative root directory */
        rpmtsSetRootDir(ts, rootdir);
    }

    /* define a dbpath macro if --dbpath option has been used */
    if(dbpath) {
        char macro[MAXMACROSIZE];
        strcpy(macro, "_dbpath ");
	strcat(macro, dbpath);
	rpmDefineMacro(NULL, macro, RMIL_CMDLINE);
    }

    (void)rpmtsSetVSFlags(ts, vsflags);

    /* open rpm's packages database */
    if( rpmtsOpenDB(ts, O_RDWR ) ) {
        printf( "Failed to open packages database\n" );
        exit(1);
    }

    return ts;

}

/*
 * Upgrade a package
 *
 * - ts		transaction set
 * - package	package file name
 */
void rpmEts_HandleUpgrade(rpmts ts, const char *package) {

    if (verbose) {
        printf("Add upgrading of package: %s\n", package);
    }

    rpmEts_AddInstallElement(ts, package, 1);

}

/*
 * Install a new package
 *
 * - ts		transaction set
 * - package	package file name
 */
void rpmEts_HandleInstall(rpmts ts, const char *package) {

    if (verbose) {
        printf("Add installation of package: %s\n", package);
    }

    rpmEts_AddInstallElement(ts, package, 0);

}

/*
 * Install or Upgrade a new package
 *
 * - ts		transaction set
 * - package	package file name
 * - upgrade	0 installation, 1 upgrading
 *
 */
void rpmEts_AddInstallElement(rpmts ts, const char *package, int upgrade) {

    Header	hdr;
    FD_t	fd;
    int		err;

    /* read package header from file */

    fd = Fopen(package, "r");

    if (fd == NULL || Ferror(fd)) {
        printf("error: cannot open file %s\n", package);
	exit(1);
    }

    rpmReadPackageFile(ts, fd, package, &hdr);

    if (hdr == NULL) {
        printf("failed to read header from package %s\n", package);
	exit(1);
    }


    Fclose(fd);

    /* add package to transaction set */

    err = rpmtsAddInstallElement(ts, hdr, strdup(package), upgrade, NULL);

    switch(err) {
        case 0: break;
        case 1: printf("I/O error reading file %s\n", package);
                exit(1);
        case 2: printf("needs capabilities to isntall package %s\n", package);
                exit(1);
    }

    headerFree(hdr);

    return;

}

/*
 * Remove a package
 *
 * - ts		transaction set
 * - package	package file name
 *
 */
void rpmEts_HandleRemove(rpmts ts, const char *package) {

    rpmdbMatchIterator	mi;
    Header		h;
    int			count;
    int			dboffset;

    if (verbose) {
        printf("Add removal of package: %s\n", package);
    }

    /* check that package is installed */

    mi = rpmtsInitIterator(ts, RPMDBI_LABEL, package, 0);
    count = rpmdbGetIteratorCount(mi);

    if (count <= 0 ) {
        printf("error: package %s is not installed\n", package);
        exit(1);
    }

    /* add operation to the transaction set */

    h = rpmdbNextIterator(mi);
    dboffset = rpmdbGetIteratorOffset(mi);

    if (rpmtsAddEraseElement(ts, h, dboffset)) {
        printf("could'n add removal of package %s to transaction\n", package);
        exit(1);
    }

    rpmdbFreeIterator(mi);

    return;

}

/*
 * Run Transaction Set
 *
 * -ts	transaction set
 */
void rpmEtsRun(rpmts ts) {

    int rc;
    int n_conflicts;
    rpmps ps;

    if (verbose) {
        printf("Run transaction set\n");
    }

    rpmtsSetFlags(ts, rpmts_flags);

    /* check for dependencies problems */

    if (rpmtsCheck(ts)) {
        printf("Failed to check dependencies\n");
        exit(1);
    }

    ps = rpmtsProblems(ts);
    n_conflicts = rpmpsNumProblems(ps);

    if (n_conflicts > 0) {
        if (!(interface_flags & INSTALL_NODEPS)) {
            rpmpsPrint(NULL, ps);
	    printf("there were %d conflicts\n", n_conflicts);
            exit(1);
	} else {
	    if (verbose) {
	        rpmpsPrint(NULL, ps);
	        printf("there were %d conflicts\n", n_conflicts);
	    }
	}
    }

    if (!(interface_flags & INSTALL_NOORDER)) {
        if (rpmtsOrder(ts)) {
            printf("error: there are packages that couldn't be ordered\n");
	    exit(1);
        }
    }

    /* run transaction set and check for problems */

    rc = rpmtsRun(ts, NULL, rpmprob_filter);
    if (rc > 0) {
        ps = rpmtsProblems(ts);
        rpmpsPrint(NULL, ps);
        printf("error running transaction set\n");
	exit(1);
    }

    rpmtsClean(ts);

    return;

}

/*
 * Clean a transaction set (free memory)
 *
 * -ts	transaction set
 */
void rpmEtsClean(rpmts ts) {

    if (verbose) {
        printf("Clean transaction set\n");
    }

    rpmtsFree(ts);

}

/*
 * Callback for rpmRunTransactions() - called at various points
 * during the installation and upgrade of packages
 */
static void *notify_CB(const void *arg, const rpmCallbackType what,
                       const unsigned long done, const unsigned long total,
                       fnpyKey key, void *data) {

    struct specfm 	*spec;
    static 		FD_t fd;
    void 		*rc;
    const char 		*filename;

    filename	= key;
    spec 	= (struct specfm *)key;
    rc 		= NULL;

    switch (what) {

        case RPMCALLBACK_INST_OPEN_FILE:
              if (debug) {
                  printf("CALLBACK_INST_OPEN_FILE\n");
              }
              fd = Fopen(filename, "r.ufdio");
              fd = fdLink(fd, "persist (showProgress)");
              return fd;
              break;
        case RPMCALLBACK_INST_CLOSE_FILE:
              if (debug) {
                  printf("CALLBACK_INST_CLOSE_FILE \n");
              }
              Fclose(fd);
              break;

        case RPMCALLBACK_INST_START:
              if (debug) {
                  printf("CALLBACK_INST_START \n");
              }
              break;
        case RPMCALLBACK_INST_PROGRESS:
              if (debug) {
                  printf("CALLBACK_INST_PROGRESS done=%ld total=%ld\n",
                         done, total);
              }
              break;

        case RPMCALLBACK_TRANS_START:
              if (debug) {
                  printf("CALLBACK_TRANS_START done=%ld total=%ld\n",
		         done, total);
              }
              break;
        case RPMCALLBACK_TRANS_PROGRESS:
              if (debug) {
                  printf("CALLBACK_TRANS_PROGRESS done=%ld total=%ld\n",
		         done, total);
              }
              break;
        case RPMCALLBACK_TRANS_STOP:
              if (debug) {
                  printf("CALLBACK_TRANS_STOP done=%ld total=%ld\n",
		         done, total);
              }
              break;

        case RPMCALLBACK_UNINST_START:
              if (debug) {
                  printf("CALLBACK_UNINST_START done=%ld total=%ld\n",
		         done, total);
              }
              break;
        case RPMCALLBACK_UNINST_PROGRESS:
              if (debug) {
                  printf("CALLBACK_UNINST_PROGRESS done=%ld total=%ld\n",
		         done, total);
              }
              break;
        case RPMCALLBACK_UNINST_STOP:
              if (debug) {
                  printf("CALLBACK_UNINST_STOP done=%ld total=%ld\n",
		         done, total);
              }
              break;

        case RPMCALLBACK_REPACKAGE_START:
	      if (debug) {
	          printf("RPMCALLBACK_REPACKAGE_START done=%ld total=%ld\n",
		  done, total);
              }
	      break;
        case RPMCALLBACK_REPACKAGE_PROGRESS:
	      if (debug) {
	          printf("RPMCALLBACK_REPACKAGE_PROGRESS done=%ld total=%ld\n",
		         done, total);
              }
	      break;
	case RPMCALLBACK_REPACKAGE_STOP:
	      if (debug) {
	          printf("RPMCALLBACK_REPACKAGE_STOP");
              }
	      break;

	case RPMCALLBACK_UNPACK_ERROR:
	      if (debug) {
	          printf("RPMCALLBACK_UNPACK_ERROR\n");
              }
	      break;
	case RPMCALLBACK_CPIO_ERROR:
	      if (debug) {
	          printf("RPMCALLBACK_CPIO_ERROR\n");
              }
	      break;

        case RPMCALLBACK_UNKNOWN:
	      if (debug) {
	          printf("RPMCALLBACK_UNKNOWN\n");
              }
	      break;

    }

    return rc;

}

/*
 * Error callback funtion, called in case of error
 */
void error_CB() {

      printf("%s", rpmErrorString());
      if (debug) {
            printf("rpm's error code number: %d\n", rpmErrorCode());
      }
      return;

}
