/*
 * rpmt.c	Transactional RPM
 *
 * rpmt is a tool to install, upgrade and erase RPM packages
 * in one transaction, thus allow package sets with complex dependencies
 * to be processed at once while keeping the system in a consistent state.
 * rpmt is part of the Quattor administration toolkit, see http://quattor.org
 * for more information about rpmt and Quattor.
 *
 * rpmt is based on RedHat's rpm and rpmlib
 * use Makefile or compile with 'gcc -Wall -lrpm rpmt.c rpmets.c -o rpmt'
 *
 * Rafael A. Garcia Leiva (angel.leiva@uam.es)
 * Universidad Autonoma de Madrid
 *
 */

#include <rpm/rpmlib.h>
#include <rpm/rpmts.h>
#include <popt.h>
#include <ctype.h>
#include <string.h>

#include "rpmt.h"

const char *parse_cmdl(int argc, const char **argv);
void parse_file(rpmts ts, const char *filename);
void print_version(void);
void print_help(poptContext poptcon);

int main(int argc, const char **argv) {

    rpmts ts;
    const char *rpmt_file = NULL;
    int ec = 0;

    /* default values for global variables */
    rcfile	= NULL,
    rootdir	= NULL,
    dbpath	= NULL;

    /* no filters or flags by default */
    interface_flags	= INSTALL_NONE;
    rpmts_flags		= RPMTRANS_FLAG_NONE;
    rpmprob_filter	= RPMPROB_FILTER_NONE;
    vsflags		= RPMVSF_DEFAULT;

    /* parse command line arguments */
    rpmt_file = parse_cmdl(argc, argv);
    if( rpmt_file == NULL ) {
        printf("usage: rpmt [options] file\n");
	exit(1);
    }

    /* show debugging information */
    if (debug) {
        verbose = 1;
        rpmSetVerbosity(RPMMESS_DEBUG);
	rpmShowRC(stdout);
    }

    /* init transaction set */
    ts = rpmEtsInit();

    /* read input file and add actions to transaction set */
    parse_file(ts, rpmt_file);

    /* run transaction set */
    rpmEtsRun(ts);

    /* clean up memory */
    rpmEtsClean(ts);

    exit(ec);

}

/*
 * Parse rpmt command line options
 *
 * rpmt accepts all the rpm's relevant options to the installation,
 * upgrading and removal of packages.
 * rpmt uses popt library to parse command line options.
 *
 */
const char *parse_cmdl(int argc, const char **argv) {

    poptContext poptcon;
    int 	rc;
    const char	*rpmt_file;

    poptcon = poptGetContext(NULL, argc, argv, optionsTable, 0);

    while( (rc = poptGetNextOpt(poptcon)) > 0 ) {

        switch (rc) {

	    /* general options */

            case POPTGENERAL_HELP:			/* --help           */
                 print_help(poptcon);
	         exit(0);
	    case POPTGENERAL_SHOWVER:			/* --version        */
                 print_version();
	         exit(0);
            case POPTGENERAL_QUIET:			/* --quiet          */
	         rpmSetVerbosity(RPMMESS_QUIET);
		 break;
            case POPTGENERAL_VERBOSE:			/* --verbose        */
	         rpmSetVerbosity(RPMMESS_VERBOSE);
		 break;

            /* bit(s) to control rpmtsRun() operation */

            case POPTTRANS_FLAG_ALLFILES:		/* --allfiles       */
	         rpmts_flags |= RPMTRANS_FLAG_ALLFILES;
		 break;
            case POPTTRANS_FLAG_NODOCS:			/* --excludedocs    */
	         rpmts_flags |= RPMTRANS_FLAG_NODOCS;
		 break;
            case POPTTRANS_FLAG_JUSTDB:			/* --justdb         */
                 rpmts_flags |= RPMTRANS_FLAG_JUSTDB;
                 break;
            case POPTTRANS_FLAG_NOSCRIPTS:		/* --noscripts      */
                 rpmts_flags |= RPMTRANS_FLAG_NOSCRIPTS;
                 break;
            case POPTTRANS_FLAG_NOTRIGGERS:		/* --notriggers     */
                 rpmts_flags |= RPMTRANS_FLAG_NOTRIGGERS;
                 break;
            case POPTTRANS_FLAG_REPACKAGE:		/* --repackage      */
	         rpmts_flags |= RPMTRANS_FLAG_REPACKAGE;
		 break;
            case POPTTRANS_FLAG_TEST:			/* --test           */
                 rpmts_flags |= RPMTRANS_FLAG_TEST;
                 break;

            /* bit(s) for filtering mechanism */

            case POPTPROB_FILTER_FORCE:			/* --force          */
	         rpmprob_filter |= RPMPROB_FILTER_OLDPACKAGE |
                                   RPMPROB_FILTER_REPLACEPKG |
                                   RPMPROB_FILTER_REPLACENEWFILES |
                                   RPMPROB_FILTER_REPLACEOLDFILES;
            case POPTPROB_FILTER_IGNORESIZE:		/* --ignoresize     */
	         rpmprob_filter |= RPMPROB_FILTER_DISKSPACE |
                                   RPMPROB_FILTER_DISKNODES;
                 break;
            case POPTPROB_FILTER_IGNOREARCH:		/* --ignorearch     */
	         rpmprob_filter |= RPMPROB_FILTER_IGNOREARCH;
		 break;
            case POPTPROB_FILTER_IGNOREOS:		/* --ignoreos       */
	         rpmprob_filter |= RPMPROB_FILTER_IGNOREOS;
		 break;
            case POPTPROB_FILTER_OLDPACKAGE:		/* --oldpackage     */
	    	 rpmprob_filter |= RPMPROB_FILTER_OLDPACKAGE;
		 break;
            case POPTPROB_FILTER_REPLACEFILES:		/* --replacefiles   */
	         rpmprob_filter |= RPMPROB_FILTER_REPLACENEWFILES |
                                   RPMPROB_FILTER_REPLACEOLDFILES;
                 break;
            case POPTPROB_FILTER_REPLACEPKG:		/* --replacepkgs    */
	         rpmprob_filter |= RPMPROB_FILTER_REPLACEPKG;
		 break;

            /* install flags */

            case POPTINSTALL_NODEPS:			/* --nodeps         */
	         interface_flags |= INSTALL_NODEPS;
		 break;
	    case POPTINSTALL_NOORDER:			/* --noorder        */
	         interface_flags |= INSTALL_NOORDER;
		 break;

            /* verify flags */

	    case POPTVERIFY_NODIGESTS:			/* --nodigest       */
	          vsflags |= _RPMVSF_NODIGESTS;
		  break;
	    case POPTVERIFY_NOSIGNATURES:		/* --nosignature    */
	          vsflags |= _RPMVSF_NOSIGNATURES;
		  break;

        }

    }

    /* last argument should be rpmt's input file name */
    rpmt_file = poptGetArg(poptcon);

    poptFreeContext(poptcon);

    return(rpmt_file);

}

/*
 * Pase rpmt imput file, and add actions to transaction set
 *
 * The format of imput file is the following:
 *
 * Empty lines and lines beginning with a # are ignored,
 * remaining lines must be in one of the following formats:
 * -i file (install package in file)
 * -u file (upgrade package in file)
 * -e package (erase package)
 *
 * TODO: there must be a better way
 */
void parse_file(rpmts ts, const char *filename) {

    FILE *fp;
    int  lineno = 0;
    char line[MAXSIZE];
    char *cp, *param;
    char cmd;

    if ( (fp = fopen(filename, "r")) == NULL ) {
        printf("open failed for %s\n", filename);
	exit(1);
    }

    while (fgets(line, MAXSIZE, fp)) {

        lineno++;
	cp = line;

        /* skip comments and empty lines */
	if (*cp == '#' ||  *cp == '\0' || *cp == '\n') {
	    continue;
	}

	/* get command (i, u or e) */
        if (*cp != '-') {
	    if(verbose) {
                printf("Syntax error on file %s at line %d\n",
		                            filename, lineno);
	    }
	    continue;
	}
        cp++;
	cmd = *cp;
	cp++;

	/* get param (package filename) */
        if (!isspace(*cp)) {
            printf("Syntax error on file %s at line %d\n", filename, lineno);
	    continue;
	}

	cp++;
        while (*cp && isspace(*cp)) {
	    cp++;
	}
        if (!*cp) {
	    if (verbose) {
	        printf("Syntax error on file %s at line %d\n", filename, lineno);
	    }
	    continue;
	}
	param = cp;
        cp = cp + strlen(cp) - 1;
        while (isspace(*cp)) {
	    *cp='\0';
	    cp--;
	}

	/* add new operation to transaction set */
        switch (cmd) {
            case 'u': /* upgrade */
	    case 'U': rpmEts_HandleUpgrade(ts, param);
	              break;
            case 'i': /* install */
            case 'I': rpmEts_HandleInstall(ts, param);
                      break;
            case 'e': /* erase */
            case 'E': rpmEts_HandleRemove(ts, param);
                      break;
            default: if (verbose) {
	                  printf("Unknown command %c on file %s at line %d\n",
	                         cmd, filename, lineno);
		      }
		      break;
        }

    }

    fclose(fp);
    return;

}

/*
 * Print rpmt version number
 */
void print_version(void) {

    printf("rpmt version %s\n", RPMT_VERSION);
    return;

}

/*
 * Print help information
 */
void print_help(poptContext poptcon) {

    printf("rpmt: a transactional RPM (version %s)\n", RPMT_VERSION);

    printf("Written by Andras Nagy <andras.nagy@cern.ch>\n");
    printf("Port to 4.0.4 by German Cancio <german.cancio@cern.ch>\n");
    printf("Re-engineered to 4.2.1 by Rafael A. Garcia Leiva");
    printf(" <angel.leiva@uam.es>\n");
    printf("Based on RPM by Red Hat Inc.\n");

    poptPrintHelp(poptcon, stdout, 0);

    return;

}
