/*
 * rpmt.h	Transactional RPM
 *
 * Header file with functions prototypes, global variables, and
 * popt structure to parse command line options.
 *
 * Rafael A. Garcia Leiva (angel.leiva@uam.es)
 * Universidad Autonoma de Madrid
 */

#ifndef H_RPMT
#define H_RPMT

#include <rpm/rpmlib.h>
#include <rpm/rpmts.h>
#include <popt.h>

#define MAXSIZE 	1024		/* rpmt input file maximum line size */
char *RPMT_VERSION = 	"4.2.1";	/* rpmt version number		     */
					/* (should match rpm version number) */

/* rpmt extensions to rpmlib */
rpmts rpmEtsInit(void);
void rpmEtsRun(rpmts ts);
void rpmEtsClean(rpmts ts);

/* rpmt internal functions */
void rpmEts_HandleUpgrade(rpmts ts, const char *package);
void rpmEts_HandleInstall(rpmts ts, const char *package);
void rpmEts_HandleRemove(rpmts ts,  const char *package);

/*
 * global variables to hanle rpmt command line options
 */
int verbose, debug;	/* control output messages			*/
char *rcfile;		/* rpm and rpmt configuration file		*/
char *rootdir;		/* top level directory for installations	*/
char *dbpath;		/* packages database path			*/

rpmInstallInterfaceFlags	interface_flags;    /* installation flags */
rpmtransFlags			rpmts_flags;	    /* transaction flags  */
rpmprobFilterFlags		rpmprob_filter;	    /* filtering flags    */
rpmVSFlags			vsflags;	    /* verify flags       */

/*
 *rpmt command line option constants
 */
#define POPTGENERAL_HELP 		1
#define POPTGENERAL_SHOWVER		2
#define POPTGENERAL_QUIET		3
#define POPTGENERAL_VERBOSE		4

#define POPTTRANS_FLAG_ALLFILES		10
#define POPTTRANS_FLAG_NODOCS		11
#define POPTTRANS_FLAG_JUSTDB		12
#define POPTTRANS_FLAG_NOSCRIPTS	13
#define POPTTRANS_FLAG_NOTRIGGERS	14
#define POPTTRANS_FLAG_REPACKAGE	15
#define POPTTRANS_FLAG_TEST		16

#define POPTPROB_FILTER_FORCE		20
#define POPTPROB_FILTER_IGNORESIZE	21
#define POPTPROB_FILTER_IGNOREARCH	22
#define POPTPROB_FILTER_IGNOREOS	23
#define POPTPROB_FILTER_OLDPACKAGE	24
#define POPTPROB_FILTER_REPLACEFILES	25
#define POPTPROB_FILTER_REPLACEPKG	26

#define POPTINSTALL_HASH		30
#define POPTINSTALL_NODEPS		31
#define POPTINSTALL_NOORDER		32
#define POPTINSTALL_PERCENT		33

#define POPTVERIFY_NODIGESTS		40
#define POPTVERIFY_NOSIGNATURES		41

/*
 * Command Line Options Table for POPT
 */
static struct poptOption optionsTable[] = {

    /* general options */

    { "help",		 '?',
      POPT_ARG_NONE,	  0,		POPTGENERAL_HELP,
      "show this help message",						0 },
    { "version",	'\0',
      POPT_ARG_NONE,	  0,		POPTGENERAL_SHOWVER,
      "print version number of rpmt being used",			0 },
    { "quiet",		'\0',
      POPT_ARG_NONE,	  0,		POPTGENERAL_QUIET,
      "provide less detailed output",					0 },
    { "verbose",	'v',
      POPT_ARG_NONE,	 &verbose,	POPTGENERAL_VERBOSE,
      "provide more detailed output",					0 },
    { "debug",		'\0',
      POPT_ARG_NONE,	 &debug,	0,
      "provide debugging information output",				0 },

    { "rcfile",		'\0',
      POPT_ARG_STRING,	&rcfile,	0,
      "read <FILE> instead of default configuration file",	 "<FILE>" },
    { "dbpath",		'\0',
      POPT_ARG_STRING,	&dbpath,	0,
      "use database in <DIR> instead of default path",		  "<DIR>" },
    { "root",		'r',
      POPT_ARG_STRING,	&rootdir,	0,
      "use ROOT as top level directory", 			   "ROOT" },

    /* installation and upgrading options */

    { "allfiles",	'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_ALLFILES,
      "install all files, even conf. which might otherwise be skipped",	0 },
    { "excludedocs",	'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_NODOCS,
      "do not install documentation",					0 },
    { "force",		'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_FORCE,
      "short hand for --replacepkgs --replacefiles",			0 },
    { "ignoresize",	'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_IGNORESIZE,
      "don't check disk space before installing",			0 },
    { "ignorearch",	'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_IGNOREARCH,
      "don't verify package architecture",				0 },
    { "ignoreos",	'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_IGNOREOS,
      "don't verify package operating system",				0 },
    { "justdb",	'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_JUSTDB,
      "update the database, but do not modify the filesystem",		0 },
    { "nodeps",	'\0',
      POPT_ARG_NONE,	  0,		POPTINSTALL_NODEPS,
      "do not verify package dependencies",				0 },
    { "noorder",	'\0',
      POPT_ARG_NONE,	  0,		POPTINSTALL_NOORDER,
      "do not reorder package installation to satisfy dependencies",	0 },
    { "noscripts",	'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_NOSCRIPTS,
      "do not execute package scriptlet(s)",				0 },
    { "notriggers",	'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_NOTRIGGERS,
      "do not execute any scriptlet(s) triggered by this package",	0 },
    { "oldpackage",	'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_OLDPACKAGE,
      "upgrade to an old version of the package",			0 },
    { "repackage",	'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_REPACKAGE,
      "re-package files before erasing",				0 },
    { "replacefiles",	'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_REPLACEFILES,
      "install even if the package replaces installed files",		0 },
    { "replacepkgs",	'\0',
      POPT_ARG_NONE,	  0,		POPTPROB_FILTER_REPLACEPKG,
      "reinstall if the package is already present",			0 },
    { "test",		'\0',
      POPT_ARG_NONE,	  0,		POPTTRANS_FLAG_TEST,
      "don't install, but tell if it would work or not",		0 },

    /* erase options */

    /* the following erase options have been already implemented:
       --nodeps, --noscripts, --notriggers, --repackage and --test */

    /* signature options */

    { "nodigest",	'\0',
      POPT_ARG_NONE,	  0,		POPTVERIFY_NODIGESTS,
      "don't verify package digest(s)",					0 },
    { "nosignature",	'\0',
      POPT_ARG_NONE,	  0,		POPTVERIFY_NOSIGNATURES,
      "don't verify package signature(s)",				0 },

    /* last entry must be null */

    { 0, 0, 0, 0, 0 }

};

#endif  /* H_RPMT */
